@@ -17,6 +17,7 @@ import six
 
 from pymemcache import pool
 
+from pymemcache.serde import LegacyWrappingSerde
 from pymemcache.exceptions import (
     MemcacheClientError,
     MemcacheUnknownCommandError,
@@ -81,7 +82,7 @@ STAT_TYPES = {
 # Common helper functions.
 
 
-def _check_key(key, allow_unicode_keys, key_prefix=b''):
+def check_key_helper(key, allow_unicode_keys, key_prefix=b''):
     """Checks key and add key_prefix."""
     if allow_unicode_keys:
         if isinstance(key, six.text_type):
@@ -144,36 +145,38 @@ class Client(object):
      just calling encode on the string (using UTF-8, for instance).
 
      If you intend to use anything but str as a value, it is a good idea to use
-     a serializer and deserializer. The pymemcache.serde library has some
-     already implemented serializers, including one that is compatible with
-     the python-memcache library.
+     a serializer. The pymemcache.serde library has an already implemented
+     serializer which pickles and unpickles data.
 
     *Serialization and Deserialization*
 
-     The constructor takes two optional functions, one for "serialization" of
-     values, and one for "deserialization". The serialization function takes
-     two arguments, a key and a value, and returns a tuple of two elements, the
-     serialized value, and an integer in the range 0-65535 (the "flags"). The
-     deserialization function takes three parameters, a key, value and flags
-     and returns the deserialized value.
+     The constructor takes an optional object, the "serializer/deserializer"
+     ("serde"), which is responsible for both serialization and deserialization
+     of objects. That object must satisfy the serializer interface by providing
+     two methods: `serialize` and `deserialize`. `serialize` takes two
+     arguments, a key and a value, and returns a tuple of two elements, the
+     serialized value, and an integer in the range 0-65535 (the "flags").
+     `deserialize` takes three parameters, a key, value, and flags, and returns
+     the deserialized value.
 
      Here is an example using JSON for non-str values:
 
      .. code-block:: python
 
-         def serialize_json(key, value):
-             if type(value) == str:
-                 return value, 1
-             return json.dumps(value), 2
+         class JSONSerde(object):
+             def serialize(self, key, value):
+                 if isinstance(value, str):
+                     return value, 1
+                 return json.dumps(value), 2
 
-         def deserialize_json(key, value, flags):
-             if flags == 1:
-                 return value
+             def deserialize(self, key, value, flags):
+                 if flags == 1:
+                     return value
 
-             if flags == 2:
-                 return json.loads(value)
+                 if flags == 2:
+                     return json.loads(value)
 
-             raise Exception("Unknown flags for value: {1}".format(flags))
+                 raise Exception("Unknown flags for value: {1}".format(flags))
 
     .. note::
 
@@ -205,6 +208,7 @@ class Client(object):
 
     def __init__(self,
                  server,
+                 serde=None,
                  serializer=None,
                  deserializer=None,
                  connect_timeout=None,
@@ -221,8 +225,9 @@ class Client(object):
 
         Args:
           server: tuple(hostname, port) or string containing a UNIX socket path.
-          serializer: optional function, see notes in the class docs.
-          deserializer: optional function, see notes in the class docs.
+          serde: optional seralizer object, see notes in the class docs.
+          serializer: deprecated serialization function
+          deserializer: deprecated deserialization function
           connect_timeout: optional float, seconds to wait for a connection to
             the memcached server. Defaults to "forever" (uses the underlying
             default socket timeout, which can be very long).
@@ -249,8 +254,7 @@ class Client(object):
           call to a method on the object will do that.
         """
         self.server = server
-        self.serializer = serializer
-        self.deserializer = deserializer
+        self.serde = serde or LegacyWrappingSerde(serializer, deserializer)
         self.connect_timeout = connect_timeout
         self.timeout = timeout
         self.no_delay = no_delay
@@ -268,8 +272,8 @@ class Client(object):
 
     def check_key(self, key):
         """Checks key and add key_prefix."""
-        return _check_key(key, allow_unicode_keys=self.allow_unicode_keys,
-                          key_prefix=self.key_prefix)
+        return check_key_helper(key, allow_unicode_keys=self.allow_unicode_keys,
+                                key_prefix=self.key_prefix)
 
     def _connect(self):
         self.close()
@@ -368,9 +372,10 @@ class Client(object):
                   flags
 
         Returns:
-          If noreply is True, the return value is always True. Otherwise the
-          return value is True if the value was stored, and False if it was
-          not (because the key already existed).
+          If ``noreply`` is True (or if it is unset and ``self.default_noreply``
+          is True), the return value is always True. Otherwise the return value
+          is True if the value was stored, and False if it was not (because
+          the key already existed).
         """
         if noreply is None:
             noreply = self.default_noreply
@@ -392,7 +397,8 @@ class Client(object):
                 flags
 
         Returns:
-          If noreply is True, always returns True. Otherwise returns True if
+          If ``noreply`` is True (or if it is unset and ``self.default_noreply``
+          is True), the return value is always True. Otherwise returns True if
           the value was stored and False if it wasn't (because the key didn't
           already exist).
         """
@@ -460,7 +466,8 @@ class Client(object):
                 flags
 
         Returns:
-          If noreply is True, always returns True. Otherwise returns None if
+          If ``noreply`` is True (or if it is unset and ``self.default_noreply``
+          is True), the return value is always True. Otherwise returns None if
           the key didn't exist, False if it existed but had a different cas
           value and True if it existed and was changed.
         """
@@ -542,7 +549,8 @@ class Client(object):
                    self.default_noreply).
 
         Returns:
-          If noreply is True, always returns True. Otherwise returns True if
+          If ``noreply`` is True (or if it is unset and ``self.default_noreply``
+          is True), the return value is always True. Otherwise returns True if
           the key was deleted, and False if it wasn't found.
         """
         if noreply is None:
@@ -602,7 +610,8 @@ class Client(object):
           value of the key, or None if the key wasn't found.
         """
         key = self.check_key(key)
-        cmd = b'incr ' + key + b' ' + six.text_type(value).encode(self.encoding)
+        value = self._check_integer(value, "value")
+        cmd = b'incr ' + key + b' ' + value
         if noreply:
             cmd += b' noreply'
         cmd += b'\r\n'
@@ -627,7 +636,8 @@ class Client(object):
           value of the key, or None if the key wasn't found.
         """
         key = self.check_key(key)
-        cmd = b'decr ' + key + b' ' + six.text_type(value).encode(self.encoding)
+        value = self._check_integer(value, "value")
+        cmd = b'decr ' + key + b' ' + value
         if noreply:
             cmd += b' noreply'
         cmd += b'\r\n'
@@ -656,9 +666,8 @@ class Client(object):
         if noreply is None:
             noreply = self.default_noreply
         key = self.check_key(key)
-        cmd = (
-            b'touch ' + key + b' ' + six.text_type(expire).encode(self.encoding)
-        )
+        expire = self._check_integer(expire, "expire")
+        cmd = b'touch ' + key + b' ' + expire
         if noreply:
             cmd += b' noreply'
         cmd += b'\r\n'
@@ -704,8 +713,8 @@ class Client(object):
         Returns:
           If no exception is raised, always returns True.
         """
-
-        self._fetch_cmd(b'cache_memlimit', [str(int(memlimit))], False)
+        memlimit = self._check_integer(memlimit, "memlimit")
+        self._fetch_cmd(b'cache_memlimit', [memlimit], False)
         return True
 
     def version(self):
@@ -739,7 +748,8 @@ class Client(object):
         """
         if noreply is None:
             noreply = self.default_noreply
-        cmd = b'flush_all ' + six.text_type(delay).encode(self.encoding)
+        delay = self._check_integer(delay, "delay")
+        cmd = b'flush_all ' + delay
         if noreply:
             cmd += b' noreply'
         cmd += b'\r\n'
@@ -772,6 +782,41 @@ class Client(object):
             error = line[line.find(b' ') + 1:]
             raise MemcacheServerError(error)
 
+    def _check_integer(self, value, name):
+        """Check that a value is an integer and encode it as a binary string"""
+        if not isinstance(value, six.integer_types):  # includes "long" on py2
+            raise MemcacheIllegalInputError(
+                '%s must be integer, got bad value: %r' % (name, value)
+            )
+
+        return six.text_type(value).encode(self.encoding)
+
+    def _check_cas(self, cas):
+        """Check that a value is a valid input for 'cas' -- either an int or a
+        string containing only 0-9
+
+        The value will be (re)encoded so that we can accept strings or bytes.
+        """
+        # convert non-binary values to binary
+        if isinstance(cas, (six.integer_types, VALID_STRING_TYPES)):
+            try:
+                cas = six.text_type(cas).encode(self.encoding)
+            except UnicodeEncodeError:
+                raise MemcacheIllegalInputError(
+                    'non-ASCII cas value: %r' % cas)
+        elif not isinstance(cas, six.binary_type):
+            raise MemcacheIllegalInputError(
+                'cas must be integer, string, or bytes, got bad value: %r' % cas
+            )
+
+        if not cas.isdigit():
+            raise MemcacheIllegalInputError(
+                'cas must only contain values in 0-9, got bad value: %r'
+                % cas
+            )
+
+        return cas
+
     def _extract_value(self, expect_cas, line, buf, remapped_keys,
                        prefixed_keys):
         """
@@ -789,8 +834,7 @@ class Client(object):
 
         buf, value = _readvalue(self.sock, buf, int(size))
         key = remapped_keys[key]
-        if self.deserializer:
-            value = self.deserializer(key, value, int(flags))
+        value = self.serde.deserialize(key, value, int(flags))
 
         if expect_cas:
             return key, (value, cas), buf
@@ -843,20 +887,18 @@ class Client(object):
 
         extra = b''
         if cas is not None:
+            cas = self._check_cas(cas)
             extra += b' ' + cas
         if noreply:
             extra += b' noreply'
-        expire = six.text_type(expire).encode(self.encoding)
+        expire = self._check_integer(expire, "expire")
 
         for key, data in six.iteritems(values):
             # must be able to reliably map responses back to the original order
             keys.append(key)
 
             key = self.check_key(key)
-            if self.serializer:
-                data, data_flags = self.serializer(key, data)
-            else:
-                data_flags = 0
+            data, data_flags = self.serde.serialize(key, data)
 
             # If 'flags' was explicitly provided, it overrides the value
             # returned by the serializer.
@@ -954,10 +996,14 @@ class PooledClient(object):
                       eventlet lock or semaphore could be used instead)
 
     Further arguments are interpreted as for :py:class:`.Client` constructor.
+
+    Note: if `serde` is given, the same object will be used for *all* clients
+    in the pool. Your serde object must therefore be thread-safe.
     """
 
     def __init__(self,
                  server,
+                 serde=None,
                  serializer=None,
                  deserializer=None,
                  connect_timeout=None,
@@ -972,8 +1018,7 @@ class PooledClient(object):
                  allow_unicode_keys=False,
                  encoding='ascii'):
         self.server = server
-        self.serializer = serializer
-        self.deserializer = deserializer
+        self.serde = serde or LegacyWrappingSerde(serializer, deserializer)
         self.connect_timeout = connect_timeout
         self.timeout = timeout
         self.no_delay = no_delay
@@ -995,13 +1040,12 @@ class PooledClient(object):
 
     def check_key(self, key):
         """Checks key and add key_prefix."""
-        return _check_key(key, allow_unicode_keys=self.allow_unicode_keys,
-                          key_prefix=self.key_prefix)
+        return check_key_helper(key, allow_unicode_keys=self.allow_unicode_keys,
+                                key_prefix=self.key_prefix)
 
     def _create_client(self):
         client = Client(self.server,
-                        serializer=self.serializer,
-                        deserializer=self.deserializer,
+                        serde=self.serde,
                         connect_timeout=self.connect_timeout,
                         timeout=self.timeout,
                         no_delay=self.no_delay,
