@@ -3,7 +3,7 @@ import time
 import logging
 import six
 
-from pymemcache.client.base import Client, PooledClient, _check_key
+from pymemcache.client.base import Client, PooledClient, check_key_helper
 from pymemcache.client.rendezvous import RendezvousHash
 from pymemcache.exceptions import MemcacheError
 
@@ -19,6 +19,7 @@ class HashClient(object):
         self,
         servers,
         hasher=RendezvousHash,
+        serde=None,
         serializer=None,
         deserializer=None,
         connect_timeout=None,
@@ -81,10 +82,12 @@ class HashClient(object):
             'no_delay': no_delay,
             'socket_module': socket_module,
             'key_prefix': key_prefix,
+            'serde': serde,
             'serializer': serializer,
             'deserializer': deserializer,
             'allow_unicode_keys': allow_unicode_keys,
             'default_noreply': default_noreply,
+            'encoding': encoding
         }
 
         if use_pooling is True:
@@ -119,7 +122,7 @@ class HashClient(object):
         self.hasher.remove_node(key)
 
     def _get_client(self, key):
-        _check_key(key, self.allow_unicode_keys, self.key_prefix)
+        check_key_helper(key, self.allow_unicode_keys, self.key_prefix)
         if len(self._dead_clients) > 0:
             current_time = time.time()
             ldc = self._last_dead_check_time
@@ -420,6 +423,9 @@ class HashClient(object):
     def replace(self, key, *args, **kwargs):
         return self._run_cmd('replace', key, False, *args, **kwargs)
 
+    def touch(self, key, *args, **kwargs):
+        return self._run_cmd('touch', key, False, *args, **kwargs)
+
     def flush_all(self):
         for _, client in self.clients.items():
             self._safely_run_func(client, client.flush_all, False)
