@@ -296,6 +296,14 @@ class ClientTestMixin(object):
         result = client.get(b'key')
         assert result is None
 
+    def test_space_key(self):
+        client = self.make_client([b''])
+        with pytest.raises(MemcacheIllegalInputError):
+            client.get(b'space key')
+
+        with pytest.raises(MemcacheIllegalInputError):
+            client.set(b'space key', b'value')
+
     def test_get_not_found_default(self):
         client = self.make_client([b'END\r\n'])
         result = client.get(b'key', default='foobar')
@@ -471,34 +479,47 @@ class TestClient(ClientTestMixin, unittest.TestCase):
         result = client.prepend(b'key', b'value', noreply=False)
         assert result is True
 
+    def test_cas_malformed(self):
+        client = self.make_client([b'STORED\r\n'])
+        with pytest.raises(MemcacheIllegalInputError):
+            client.cas(b'key', b'value', 'nonintegerstring', noreply=False)
+
+        with pytest.raises(MemcacheIllegalInputError):
+            # even a space makes it a noninteger string
+            client.cas(b'key', b'value', '123 ', noreply=False)
+
+        with pytest.raises(MemcacheIllegalInputError):
+            # non-ASCII digit
+            client.cas(b'key', b'value', u'⁰', noreply=False)
+
     def test_cas_stored(self):
         client = self.make_client([b'STORED\r\n'])
-        result = client.cas(b'key', b'value', b'cas', noreply=False)
+        result = client.cas(b'key', b'value', b'123', noreply=False)
         assert result is True
 
         # unit test for encoding passed in __init__()
         client = self.make_client([b'STORED\r\n'], encoding='utf-8')
-        result = client.cas(b'key', b'value', b'cas', noreply=False)
+        result = client.cas(b'key', b'value', b'123', noreply=False)
         assert result is True
 
     def test_cas_exists(self):
         client = self.make_client([b'EXISTS\r\n'])
-        result = client.cas(b'key', b'value', b'cas', noreply=False)
+        result = client.cas(b'key', b'value', b'123', noreply=False)
         assert result is False
 
         # unit test for encoding passed in __init__()
         client = self.make_client([b'EXISTS\r\n'], encoding='utf-8')
-        result = client.cas(b'key', b'value', b'cas', noreply=False)
+        result = client.cas(b'key', b'value', b'123', noreply=False)
         assert result is False
 
     def test_cas_not_found(self):
         client = self.make_client([b'NOT_FOUND\r\n'])
-        result = client.cas(b'key', b'value', b'cas', noreply=False)
+        result = client.cas(b'key', b'value', b'123', noreply=False)
         assert result is None
 
         # unit test for encoding passed in __init__()
         client = self.make_client([b'NOT_FOUND\r\n'], encoding='utf-8')
-        result = client.cas(b'key', b'value', b'cas', noreply=False)
+        result = client.cas(b'key', b'value', b'123', noreply=False)
         assert result is None
 
     def test_cr_nl_boundaries(self):
@@ -665,10 +686,14 @@ class TestClient(ClientTestMixin, unittest.TestCase):
         assert result is False
 
     def test_serialization(self):
-        def _ser(key, value):
-            return json.dumps(value), 0
+        class JsonSerde(object):
+            def serialize(self, key, value):
+                return json.dumps(value).encode('ascii'), 0
 
-        client = self.make_client([b'STORED\r\n'], serializer=_ser)
+            def deserialize(self, key, value, flags):
+                return json.loads(value.decode('ascii'))
+
+        client = self.make_client([b'STORED\r\n'], serde=JsonSerde())
         client.set('key', {'c': 'd'})
         assert client.sock.send_bufs == [
             b'set key 0 0 10 noreply\r\n{"c": "d"}\r\n'
@@ -797,6 +822,23 @@ class TestClient(ClientTestMixin, unittest.TestCase):
         with pytest.raises(MemcacheIllegalInputError):
             _set()
 
+    def test_set_key_with_noninteger_expire(self):
+        client = self.make_client([b''])
+
+        class _OneLike(object):
+            """object that looks similar to the int 1"""
+            def __str__(self):
+                return "1"
+
+        for noreply in (True, False):
+            for expire in (1.5, _OneLike(), "1"):
+                def _set():
+                    client.set(b'finekey', b'finevalue',
+                               noreply=noreply, expire=expire)
+
+                with pytest.raises(MemcacheIllegalInputError):
+                    _set()
+
     def test_set_many_socket_handling(self):
         client = self.make_client([b'STORED\r\n'])
         result = client.set_many({b'key': b'value'}, noreply=False)
@@ -1188,22 +1230,23 @@ class TestMockClient(ClientTestMixin, unittest.TestCase):
         assert result == b'value'
 
     def test_deserialization(self):
-        def _serializer(key, value):
-            if isinstance(value, dict):
-                return json.dumps(value).encode('UTF-8'), 1
-            return value, 0
+        class JsonSerde(object):
+            def serialize(self, key, value):
+                if isinstance(value, dict):
+                    return json.dumps(value).encode('UTF-8'), 1
+                return value, 0
 
-        def _deserializer(key, value, flags):
-            if flags == 1:
-                return json.loads(value.decode('UTF-8'))
-            return value
+            def deserialize(self, key, value, flags):
+                if flags == 1:
+                    return json.loads(value.decode('UTF-8'))
+                return value
 
         client = self.make_client([
             b'STORED\r\n',
             b'VALUE key1 0 5\r\nhello\r\nEND\r\n',
             b'STORED\r\n',
             b'VALUE key2 0 18\r\n{"hello": "world"}\r\nEND\r\n',
-        ], serializer=_serializer, deserializer=_deserializer)
+        ], serde=JsonSerde())
 
         result = client.set(b'key1', b'hello', noreply=False)
         result = client.get(b'key1')
